% Supplamentay material to the paper:
% Linear Characteristic Graphical Models: Representation, Inference and
% Applications. By Danny Bickson and Carlos Guestrin. In NIPS 2010.
% http://arxiv.org/abs/1008.5325
% 
% 
% Written by Danny Bickson, CMU
% Source code is available from http://www.cs.cmu.edu/~bickson/stable/
%
% Stable Addition
% Using proposition 1.16 of NOLAN's stable distribution book


function [s] = add_stable(ss,prior)
        first = ss{1};
        s(1) = first(1);
        alpha = s(1);
        assert(alpha >= 0 && alpha <= 2);
        gamma = prior(3)^alpha;
        beta = prior(2)*prior(3)^alpha;
        delta = prior(4);
        assert(alpha == prior(1));
        
        for i=1:length(ss)
            elem = ss{i};
            gamma = gamma + (elem(3)^alpha);
            beta = beta + elem(2)*elem(3)^alpha;
            delta = delta + elem(4);
        end
        assert(gamma > 0);
        beta = beta / gamma;
        assert(beta >= -1 && beta <= 1);
        
        gamma = gamma^(1/alpha);
        
      
        if (s(1) ~= 1) %alpha != 1
            fact = tan((s(1)*pi) / 2);
            tsum = fact*beta*gamma;
            for i=1:length(ss)
                elem = ss{i};
                tsum = tsum - fact*elem(2)*elem(3); 
            end
        else %alpha = 1
            fact = 2/pi;
            tsum = fact*beta*gamma*log(gamma);
            for i=1:length(ss)
               elem = ss{i};
               assert(elem(3) > 0);
               tsum = tsum - fact*elem(2)*elem(3)*log(elem(3));
            end
            
        end
            
        delta = delta+tsum;
        s(4) = delta;
        s(3) = gamma;
        s(2) = beta;
     
        if ~(beta >= -1 && beta <= 1);
            disp('bug');
        end
        assert(gamma > 0);
       
    end