% Supplamentay material to the paper:
% Linear Characteristic Graphical Models: Representation, Inference and
% Applications. By Danny Bickson and Carlos Guestrin. In NIPS 2010.
% http://arxiv.org/abs/1008.5325
% 
% 
% Written by Danny Bickson, CMU
% Source code is available from http://www.cs.cmu.edu/~bickson/stable/
%
% This example shows how the exact computation can retrewive a hidden
% vector of stable variables X, where the observation Y=AX is table.

A=[0 0.2 0.3; -0.2 0 -0.3 ; 0.6 0.1 0];
x=[-1 1 -1];
alpha=1;
%beta=[1 0.5 0 1];
beta = [ 0 0.5 1];
gamma = 1*[1 1 1];
n = 3;
addpath ..

s=cell(2,1);
 for i=1:n
     elem(1)=alpha;
     elem(2)=beta(i);
     elem(3)=gamma(i);
     elem(4)=x(i);
     s{i} =elem;
 end
 
 prior(1) = alpha;
 prior(2) = 0;
 prior(3) = 0;
 prior(4) = 0;

 % compute linear transformation Y=AX
 ret = stable_mult_mat(A,s,prior);
 y=A*x';
 
 % compute the inverse transformation X=A^{-1}Y
  out = stable_inv_mult_mat(A,ret,prior);
 
 for i=1:n
    for j=1:n
      %Verify computation is exact
      assert(norm(out{i}(j)-s{i}(j))<1e-8);
    end
    
    disp(num2str(out{i}));
    disp(num2str(s{i}));
    disp('************************************');
 end