% Supplamentay material to the paper:
% Linear Characteristic Graphical Models: Representation, Inference and
% Applications. By Danny Bickson and Carlos Guestrin. In NIPS 2010.
% http://arxiv.org/abs/1008.5325
% 
% 
% Written by Danny Bickson, CMU
% Source code is available from http://www.cs.cmu.edu/~bickson/stable/
%
% This example shows how the exact computation can retrewive a hidden
% vector of stable variables X, where the observation Y=AX is table.
% This sample tests the case where alpha=1

A=[1 0.2 0.3; -0.2 1 -0.3 ; 0.6 0.1 1];
x=[-1 1 -1];
alpha=0.8;
beta = [ 0 0.5 1];
gamma = 1*[1 .5 .2];
n = 3;
addpath ..

s=cell(2,1);
 for i=1:n
     elem(1)=alpha;
     elem(2)=beta(i);
     elem(3)=gamma(i);
     elem(4)=x(i);
     s{i} =elem;
 end
 
 prior(1) = alpha;
 prior(2) = 0;
 prior(3) = 1;
 prior(4) = 0;
 % output gamma is ((eye(3)+abs(A).^alpha)*gamma'.^alpha).^(1/alpha)
 ret = stable_mult_mat(A,s,prior);
 y=A*x';
 
  out = stable_inv_mult_mat(A,ret,prior);
 
  out2 = stable_jacobi2a(A,ret,60,1e-5,prior);
  

 for i=1:n
    for j=1:4
       assert(norm(out{i}(j)-s{i}(j))<1e-8);
    end
    disp(['exact solution ', num2str(out{i})]);
    disp(['approx solution:', num2str(out2{i})]);
    disp(['should be:', num2str(s{i})]);
    disp('************************************');
 end