% Supplamentay material to the paper:
% Linear Characteristic Graphical Models: Representation, Inference and
% Applications. By Danny Bickson and Carlos Guestrin. In NIPS 2010.
% http://arxiv.org/abs/1008.5325
% 
% 
% Written by Danny Bickson, CMU
% Source code is available from http://www.cs.cmu.edu/~bickson/stable/
%
% Inverse transform of stable linear channel


function [vout]=stable_inv_mult_mat(A,vin,sigmaprior)
    assert(sum(sum(isnan(A))) == 0);
    assert(sum(sum(~isreal(A))) == 0);
    assert(sum(sum(~isfinite(A))) == 0);
  
    n=length(A);
    alpha = vin{1}(1);
    assert(alpha>0 && alpha<=2);
    vout = cell(n,1);
    
  
    gamma = zeros(n,1);
    beta = zeros(n,1);
    delta = zeros(n,1);
    pdiag = zeros(n,1);
    
    for i=1:n
       gamma(i,1) = vin{i}(3); 
       beta(i,1) = vin{i}(2);
       delta(i,1) = vin{i}(4);
       pdiag(i,1) = sigmaprior(3)^alpha;
    end
    
    gammaout = inv(abs(A).^alpha)*(gamma.^alpha - pdiag); 
    betaout = (inv(sign(A).*abs(A).^alpha)*(beta.*gamma.^alpha - sigmaprior(2).*sigmaprior(3).^alpha))./gammaout;
  
    
    gammaout = gammaout.^(1/alpha);
    
    deltaout = delta - sigmaprior(4);
    if (alpha ~= 1)
       deltaout = deltaout - tan(0.5*pi*alpha)*(beta.*gamma - A*(betaout.*gammaout)); 
       deltaout = inv(A)*deltaout;
    else
       B=A.*log(abs(A));
       B(A==0)=0; 
       deltaout = deltaout - (2/pi)*(beta.*gamma.*log(gamma) - B*(betaout.*gammaout) - A*(betaout.*gammaout.*log(gammaout)));
       deltaout = inv(A)*deltaout;
    end
        
    
%     disp('Gamma');
%     disp(num2str(gamma'));
%     disp(num2str(gammaout'));
%     disp('Beta');
%     disp(num2str(beta'));
%     disp(num2str(betaout'));
%     disp('Delta');
%     disp(num2str(delta'));
%     disp(num2str(deltaout'));
    for i=1:n
        vout{i}(2) = betaout(i);
        vout{i}(3) = gammaout(i);
        vout{i}(4) = deltaout(i);
        vout{i}(1) = alpha;
    end
end