% Supplamentay material to the paper:
% Linear Characteristic Graphical Models: Representation, Inference and
% Applications. By Danny Bickson and Carlos Guestrin. In NIPS 2010.
% http://arxiv.org/abs/1008.5325
% 
% 
% Written by Danny Bickson, CMU
% Source code is available from http://www.cs.cmu.edu/~bickson/stable/

% This code multiplties a matrix of scalars with a vector of stable random
% variales. See equation 1.8 in Chapter 1 in Nolan's book:
% http://academic2.american.edu/~jpnolan/stable/chap1.pdf
% output: a stable distribution

function [outbeta, outgamma, outdelta] = stable_mult_mat_block(A, alpha, beta, gamma, delta)
   
   assert(alpha>0 && alpha <= 2);
   assert(length(beta) == length(A));
   assert(min(gamma) > 0);
   
   outgamma = ((abs(A).^alpha) * gamma.^alpha);
   outbeta = ((sign(A).*abs(A).^alpha)*(beta.*gamma.^alpha))./ outgamma;
   outgamma = outgamma.^(1/alpha);
   outdelta = A*delta;
   if (alpha ~= 1)
      outdelta = outdelta + tan(pi*alpha/2)*(outbeta.*outgamma - A*(beta.*gamma));
   else
      assert(min(outgamma) > 0);
      outdelta = outdelta + (2/pi)*(outbeta.*outgamma.*log(outgamma) - A*(beta.*gamma.*log(gamma))-A*log(A)*(beta.*gamma));
   end
   


end