% Supplamentay material to the paper:
% Linear Characteristic Graphical Models: Representation, Inference and
% Applications. By Danny Bickson and Carlos Guestrin. In NIPS 2010.
% http://arxiv.org/abs/1008.5325
% 
% 
% Written by Danny Bickson, CMU
% Source code is available from http://www.cs.cmu.edu/~bickson/stable/

% This code tests that the linear transformation is done correctly
% output: a stable distribution

function [] =test_stable_mult_mat()
    rand('seed', 10);
    A = rand(3,3);
    gamma = [1 1 2]';
    delta = [2 1 -8]';
    beta = [0 0.5 -1]';
    alpha = 0.9;
    
    s=cell(3,1);
    for i=1:3
       s{i} = [ alpha beta(i) gamma(i) delta(i)];
    end
    prior = [alpha 0 0 0];
    
    ret = stable_mult_mat(A, s, prior);
    
    [outbeta, outgamma, outdelta] = stable_mult_mat_block(A, alpha, beta, gamma, delta);
    
    for i=1:3
        assert(norm(ret{i}(2) - outbeta(i))<1e-10);
        assert(norm(ret{i}(3) - outgamma(i))<1e-10);
        assert(norm(ret{i}(4) - outdelta(i))<1e-10);
    end


end